#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2020 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of Pandora plugin LibSGM
#
#     https://github.com/CNES/Pandora_plugin_libsgm
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module provides functions to test Pandora + plugin_LibSGM
"""
import numpy as np

import pandora
from pandora.state_machine import PandoraMachine
from tests import common


class TestPluginPythonParall:
    """
    TestPlugin class allows to test pandora + plugin_libsgm
    """

    def test_libsgm(self, left_cones, right_cones, disp_left, disp_right):
        """
        Test pandora + plugin_libsgm

        """
        user_cfg = pandora.read_config_file("tests/conf/sgm_python_parall.json")

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Import pandora plugins
        pandora.import_plugin()

        # Run the pandora pipeline
        left, right = pandora.run(pandora_machine, left_cones, right_cones, -60, 0, user_cfg)

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 1, flag_inverse_value=False) <= 0.20

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 2, flag_inverse_value=False) <= 0.15

        # Check the left validity mask cross checking ( bit 8 and 9 )
        # Compares the calculated validity mask with the ground truth ( occlusion mask )
        occlusion = np.ones((left["validity_mask"].shape[0], left["validity_mask"].shape[1]))
        occlusion[left["validity_mask"].data >= 512] = 0

        # If the percentage of errors is > 0.15, raise an error
        assert common.error_mask(occlusion, occlusion) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 2) <= 0.15

    def test_libsgm_negative_disparities(self, left_cones, right_cones, disp_left, disp_right):
        """
        Test pandora + plugin_libsgm, with negative disparities

        """
        user_cfg = pandora.read_config_file("tests/conf/sgm_python_parall.json")

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Run the pandora pipeline
        left, right = pandora.run(pandora_machine, left_cones, right_cones, -60, -1, user_cfg)

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 1, flag_inverse_value=False) <= 0.20

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 2, flag_inverse_value=False) <= 0.15

        # Check the left validity mask cross checking ( bit 8 and 9 )
        # Compares the calculated validity mask with the ground truth ( occlusion mask )
        occlusion = np.ones((left["validity_mask"].shape[0], left["validity_mask"].shape[1]))
        occlusion[left["validity_mask"].data >= 512] = 0

        # If the percentage of errors is > 0.15, raise an error
        assert common.error_mask(occlusion, occlusion) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 2) <= 0.15

    def test_libsgm_positive_disparities(self, left_cones, right_cones, disp_left, disp_right):
        """
        Test pandora + plugin_libsgm, with positive disparities

        """
        user_cfg = pandora.read_config_file("tests/conf/sgm_python_parall.json")

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        right, left = pandora.run(pandora_machine, right_cones, left_cones, 1, 60, user_cfg)

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 1, flag_inverse_value=False) <= 0.20

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 2, flag_inverse_value=False) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 2) <= 0.15

    def test_libsgm_zncc(self, left_cones, right_cones, disp_left_zncc, disp_right_zncc):
        """
        Test pandora + plugin_libsgm if ZNCC measure is used
        """

        # Prepare the configuration
        user_cfg = pandora.read_config_file("tests/conf/sgm_zncc_python_parall.json")

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Run the pandora pipeline
        left, right = pandora.run(pandora_machine, left_cones, right_cones, -60, 0, user_cfg)

        # Compares the calculated left disparity map with the ground truth
        # If the disparity maps are not equal, raise an error

        np.testing.assert_allclose(left["disparity_map"].data, disp_left_zncc, rtol=1e-04)

        # Compares the calculated right disparity map with the ground truth
        # If the disparity maps are not equal, raise an error
        np.testing.assert_allclose(right["disparity_map"].data, disp_right_zncc, rtol=1e-04)



# Copyright (c) 2023 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of Pandora plugin LibSGM
#
#     https://github.com/CNES/Pandora_plugin_libsgm
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Fixtures."""
# pylint: disable=redefined-outer-name

import pandora
import pytest

import numpy as np
import rasterio
import xarray as xr


@pytest.fixture()
def inputs_path(resource_path_root):
    return resource_path_root / "inputs"


@pytest.fixture()
def outputs_path(resource_path_root):
    return resource_path_root / "outputs"


@pytest.fixture()
def configurations_path(resource_path_root):
    return resource_path_root / "conf"


@pytest.fixture()
def left_cones(inputs_path):
    """Cones images."""
    return pandora.create_dataset_from_inputs(
        {"img": str(inputs_path / "left.png"), "nodata": np.nan, "mask": None, "disp": [-60, 0]}
    )


@pytest.fixture()
def right_cones(inputs_path):
    """Cones images."""
    return pandora.create_dataset_from_inputs(
        {"img": str(inputs_path / "right.png"), "nodata": np.nan, "mask": None, "disp": [0, 60]}
    )


@pytest.fixture()
def left_rgb(inputs_path):
    """Cones images."""
    return pandora.create_dataset_from_inputs(
        {"img": str(inputs_path / "left_rgb.tif"), "nodata": np.nan, "mask": None, "disp": [-60, 0]}
    )


@pytest.fixture()
def right_rgb(inputs_path):
    """Cones images."""
    return pandora.create_dataset_from_inputs(
        {"img": str(inputs_path / "right_rgb.tif"), "nodata": np.nan, "mask": None}
    )


@pytest.fixture()
def left_cones_classif(inputs_path):
    """Cones images with classification."""
    return pandora.create_dataset_from_inputs(
        {
            "img": str(inputs_path / "left.png"),
            "nodata": np.nan,
            "mask": None,
            "classif": str(inputs_path / "left_classif.tif"),
            "disp": [-60, 0],
        }
    )


@pytest.fixture()
def right_cones_classif(inputs_path):
    """Cones images with classification."""
    return pandora.create_dataset_from_inputs(
        {
            "img": str(inputs_path / "right.png"),
            "nodata": np.nan,
            "mask": None,
            "classif": str(inputs_path / "right_classif.tif"),
        }
    )


@pytest.fixture()
def left_cones_segm(inputs_path):
    """Cones images with segmentation."""
    return pandora.create_dataset_from_inputs(
        {
            "img": str(inputs_path / "left.png"),
            "nodata": np.nan,
            "mask": None,
            "segm": str(inputs_path / "left_classif.tif"),
            "disp": [-60, 0],
        }
    )


@pytest.fixture()
def right_cones_segm(inputs_path):
    """Cones images with segmentation."""
    return pandora.create_dataset_from_inputs(
        {
            "img": str(inputs_path / "right.png"),
            "nodata": np.nan,
            "mask": None,
            "segm": str(inputs_path / "right_classif.tif"),
        }
    )


@pytest.fixture()
def disp_left(outputs_path):
    return rasterio.open(outputs_path / "disp_left.tif").read(1)


@pytest.fixture()
def disp_right(outputs_path):
    return rasterio.open(outputs_path / "disp_right.tif").read(1)


@pytest.fixture()
def occlusion(outputs_path):
    return rasterio.open(outputs_path / "occl.png").read(1)


@pytest.fixture()
def disp_left_zncc(outputs_path):
    return rasterio.open(outputs_path / "disp_left_zncc.tif").read(1)


@pytest.fixture()
def disp_right_zncc(outputs_path):
    return rasterio.open(outputs_path / "disp_right_zncc.tif").read(1)


@pytest.fixture()
def left_crafted():
    """Manually computed images."""
    data = np.array(
        ([1, 1, 1, 1, 1], [1, 1, 1, 1, 2], [1, 1, 1, 4, 3], [1, 1, 1, 1, 1]),
        dtype=np.float32,
    )
    result = xr.Dataset(
        {"im": (["row", "col"], data)},
        coords={"row": np.arange(data.shape[0]), "col": np.arange(data.shape[1])},
        attrs={
            "no_data_img": 0,
            "valid_pixels": 0,
            "no_data_mask": 1,
            "crs": None,
            "transform": None,
        },
    )
    return result


@pytest.fixture()
def right_crafted():
    """Manually computed images."""
    data = np.array(
        ([1, 1, 1, 2, 2], [1, 1, 1, 4, 2], [1, 1, 1, 4, 4], [1, 1, 1, 1, 1]),
        dtype=np.float32,
    )
    result = xr.Dataset(
        {"im": (["row", "col"], data)},
        coords={"row": np.arange(data.shape[0]), "col": np.arange(data.shape[1])},
        attrs={
            "no_data_img": 0,
            "valid_pixels": 0,
            "no_data_mask": 1,
            "crs": None,
            "transform": None,
        },
    )
    return result



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2020 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of Pandora plugin LibSGM
#
#     https://github.com/CNES/Pandora_plugin_libsgm
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module provides functions to test Pandora + plugin_LibSGM
"""

import copy
import pytest
import numpy as np
import pandora
import rasterio
import xarray as xr
from pandora import optimization, check_conf, check_configuration
from pandora.state_machine import PandoraMachine
from tests import common

# pylint: disable=too-many-lines, too-many-public-methods, redefined-outer-name


@pytest.fixture()
def cost_volume():
    """Create cost volume."""
    data_cv = np.array(
        [
            [[1, 1, 1, 1, 1], [1, 1, 1, 1, 2], [1, 1, 1, 4, 3], [1, 1, 1, 1, 1]],
            [[1, 1, 2, 1, 1], [1, 1, 1, 1, 2], [1, 1, 1, 4, 3], [1, 1, 1, 1, 7]],
            [[1, 4, 1, 1, 1], [1, 1, 1, 1, 2], [1, 1, 1, 4, 3], [1, 12, 1, 1, 1]],
        ],
        dtype=np.float32,
    )
    result = xr.Dataset(
        {"cost_volume": (["row", "col", "disp"], data_cv)},
        coords={
            "row": np.arange(data_cv.shape[0]),
            "col": np.arange(data_cv.shape[1]),
            "disp": np.arange(data_cv.shape[2]),
            "indicator": ["confidence_from_ambiguity"],
        },
        attrs={
            "no_data_img": 0,
            "valid_pixels": 0,
            "no_data_mask": 1,
            "crs": None,
            "transform": None,
        },
    )
    return result


@pytest.fixture()
def user_cfg(configurations_path):
    return pandora.read_config_file(str(configurations_path / "3sgm.json"))


@pytest.fixture()
def inputs_with_classif(inputs_path):
    return {
        "img_left": str(inputs_path / "left.png"),
        "left_classif": str(inputs_path / "left_classif.tif"),
        "disp_left": [-60, 0],
        "nodata_left": "NaN",
        "img_right": str(inputs_path / "right.png"),
        "right_classif": str(inputs_path / "right_classif.tif"),
        "nodata_right": "NaN",
    }


@pytest.fixture()
def inputs_with_segment(inputs_path):
    return {
        "img_left": str(inputs_path / "left.png"),
        "left_segm": str(inputs_path / "left_classif.tif"),
        "disp_left": [-60, 0],
        "nodata_left": "NaN",
        "img_right": str(inputs_path / "right.png"),
        "right_segm": str(inputs_path / "right_classif.tif"),
        "nodata_right": "NaN",
    }


class TestPlugin3SGM:
    """
    TestPlugin class allows to test pandora + plugin_lib3sgm
    """

    def test_lib3sgm(self, left_cones, right_cones, disp_left, disp_right, user_cfg):
        """
        Test pandora + plugin_lib3sgm
        """
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Run the pandora pipeline
        left, right = pandora.run(pandora_machine, left_cones, right_cones, user_cfg)

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 1, flag_inverse_value=False) <= 0.20

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 2, flag_inverse_value=False) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 2) <= 0.15

    def test_compute_optimization_layer_none_layer_in_dataset(self, left_crafted, cost_volume, user_cfg):
        """
        Test plugin_libsgm compute_optimization_layer function,
        with user asking for piecewise optimization, without any in dataset
        """

        # Prepare the configuration
        # add internal geometric_prior source
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "internal"}

        # Import pandora plugins
        pandora.import_plugin()

        # Load plugins
        optimization_ = optimization.AbstractOptimization(left_crafted, **user_cfg["pipeline"]["optimization"])

        classif_arr = optimization_.compute_optimization_layer(cost_volume, left_crafted, left_crafted["im"].data.shape)

        gt_classif = np.ones((4, 5))
        np.testing.assert_array_equal(classif_arr, gt_classif)

    def test_optimization_layer_with_3sgm(self, cost_volume, user_cfg):
        """
        Test the optimization layer function with 3sgm default configuration
        """

        # Import pandora plugins
        pandora.import_plugin()

        data = np.array(([1, 1, 1, 1], [1, 1, 2, 1], [1, 1, 4, 1]), dtype=np.float32)
        left = xr.Dataset(
            {"im": (["row", "col"], data)},
            coords={"row": np.arange(data.shape[0]), "col": np.arange(data.shape[1])},
            attrs={
                "no_data_img": 0,
                "valid_pixels": 0,
                "no_data_mask": 1,
                "crs": None,
                "transform": None,
            },
        )
        gt_default_prior_array = np.ones(left["im"].shape, dtype=np.float32)

        # Load plugins
        optimization_ = optimization.AbstractOptimization(left, **user_cfg["pipeline"]["optimization"])

        cv_in = copy.deepcopy(cost_volume)

        prior_array_out = optimization_.compute_optimization_layer(cv_in, left, left["im"].data.shape)

        # check that added array in cv is correct
        np.testing.assert_array_equal(cv_in["internal"], gt_default_prior_array)

        # Check that prior array is the default one
        np.testing.assert_array_equal(gt_default_prior_array, prior_array_out)

    def test_user_initiate_3sgm_with_geomprior_internal(self, left_cones, right_cones, disp_left, disp_right, user_cfg):
        """
        Test that user can implement internal geometric_prior with a 3sgm configuration
        """

        # Prepare the SGM configuration
        # Add a geometric_prior
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "internal"}
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Run the pandora pipeline
        left, right = pandora.run(pandora_machine, left_cones, right_cones, user_cfg)

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 1, flag_inverse_value=False) <= 0.20

        # Compares the calculated left disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, disp_left, 2, flag_inverse_value=False) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, disp_right, 2) <= 0.15

    def test_user_initiate_3sgm_with_none_geomprior_classif(self, left_cones, right_cones, user_cfg):
        """
        Test that user can't implement classif geometric_prior with a 3sgm configuration
        and no classification in image dataset
        """

        # Prepare the SGM configuration
        # Add a geometric_prior
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "classif"}
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Pandora pipeline should fail
        with pytest.raises(SystemExit):
            _, _ = pandora.run(pandora_machine, left_cones, right_cones, user_cfg)

    def test_user_initiate_3sgm_with_none_geomprior_segmentation(self, left_cones, right_cones, user_cfg):
        """
        Test that user can't implement segmentation geometric_prior with a 3sgm configuration
        and no segmentation in image dataset
        """

        # Prepare the SGM configuration
        # Add a geometric_prior
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Pandora pipeline should fail
        with pytest.raises(SystemExit):
            _, _ = pandora.run(pandora_machine, left_cones, right_cones, user_cfg)

    @staticmethod
    def test_user_initiate_3sgm_with_geomprior_segmentation(user_cfg, inputs_path, outputs_path):
        """
        Test that user can implement segmentation geometric_prior with a 3sgm configuration
        and segmentation in image dataset
        """

        # Add mask to left data
        masked_left = pandora.create_dataset_from_inputs(
            {
                "img": str(inputs_path / "left.png"),
                "nodata": np.nan,
                "mask": None,
                "segm": str(inputs_path / "white_band_mask.png"),
                "disp": [-60, 0],
            }
        )
        masked_right = pandora.create_dataset_from_inputs(
            {
                "img": str(inputs_path / "right.png"),
                "nodata": np.nan,
                "mask": None,
                "segm": str(inputs_path / "white_band_mask.png"),
            }
        )

        # Ground truth
        gt_left = rasterio.open(outputs_path / "left_disparity_3sgm.tif").read(1)
        gt_right = rasterio.open(outputs_path / "right_disparity_3sgm.tif").read(1)

        # Add a geometric_prior
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        left, right = pandora.run(pandora_machine, masked_left, masked_right, user_cfg)

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 2) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, gt_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, gt_right, 2) <= 0.15

    def test_classif_on_right_and_left_with_one_class(
        self, left_cones_classif, right_cones_classif, user_cfg, inputs_with_classif, outputs_path
    ):
        """
        Optimization on one existing band for left and right classification with validation step.
        """

        # Prepare the SGM configuration
        # Add a classification and one class
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["olive tree"],
        }

        # Add inputs
        user_cfg["input"] = inputs_with_classif

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        user_cfg = check_conf(user_cfg, pandora_machine)

        left, right = pandora.run(pandora_machine, left_cones_classif, right_cones_classif, user_cfg)

        # Ground truth
        gt_left = rasterio.open(outputs_path / "left_disparity_3sgm.tif").read(1)
        gt_right = rasterio.open(outputs_path / "right_disparity_3sgm.tif").read(1)

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 2) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, gt_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, gt_right, 2) <= 0.15

    def test_classif_on_right_and_left_with_two_classes(
        self, left_cones_classif, right_cones_classif, user_cfg, inputs_with_classif, outputs_path
    ):
        """
        Optimization on two existing bands for left and right classification with validation step.
        """

        # Prepare the SGM configuration
        # Add a classification and two classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["olive tree", "cornfields"],
        }

        # Add inputs
        user_cfg["input"] = inputs_with_classif

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        user_cfg = check_conf(user_cfg, pandora_machine)

        left, right = pandora.run(pandora_machine, left_cones_classif, right_cones_classif, user_cfg)

        # Ground truth
        gt_left = rasterio.open(outputs_path / "left_disparity_3sgm.tif").read(1)
        gt_right = rasterio.open(outputs_path / "right_disparity_3sgm.tif").read(1)

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 2) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, gt_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, gt_right, 2) <= 0.15

    def test_classif_on_right_and_left_with_wrong_class(self, user_cfg, inputs_with_classif):
        """
        Optimization on wrong band for left and right classification with validation step.
        "peuplier" band doesn't exists.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and a wrong class
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["peuplier"],
        }

        # Add inputs
        user_cfg["input"] = inputs_with_classif

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_classif_on_right_and_left_with_no_class(self, user_cfg, inputs_with_classif):
        """
        Optimization for left and right classification with wrong configuration with validation step.
        Classes are required for source as "classif" in geometric_prior.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "classif"}

        # Add inputs
        user_cfg["input"] = inputs_with_classif

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_classif_on_right_with_validation(self, user_cfg, inputs_with_classif):
        """
        Optimization with only right classification present
        Validation step requires both left and right classifications
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["cornfields"],
        }
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Add inputs
        user_cfg["input"] = inputs_with_classif
        del user_cfg["input"]["left_classif"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_classif_on_left_with_validation(self, user_cfg, inputs_with_classif):
        """
        Optimization for left classification with validation step.
        Validation step requires both left and right classifications
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["cornfields"],
        }
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Add inputs
        user_cfg["input"] = inputs_with_classif
        del user_cfg["input"]["right_classif"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_segm_on_right_and_left(
        self, left_cones_segm, right_cones_segm, user_cfg, inputs_with_segment, outputs_path
    ):
        """
        Optimization left and right segmentation with validation step.
        """

        # Prepare the SGM configuration
        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}

        # Add inputs
        user_cfg["input"] = inputs_with_segment

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        user_cfg = check_conf(user_cfg, pandora_machine)

        left, right = pandora.run(pandora_machine, left_cones_segm, right_cones_segm, user_cfg)

        # Ground truth
        gt_left = rasterio.open(outputs_path / "left_disparity_3sgm.tif").read(1)
        gt_right = rasterio.open(outputs_path / "right_disparity_3sgm.tif").read(1)

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 2) <= 0.15

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(right["disparity_map"].data, gt_right, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(right["disparity_map"].data, gt_right, 2) <= 0.15

    def test_segm_with_classes(self, user_cfg, inputs_with_classif):
        """
        Optimization left and right segmentation with validation step and classes instantiated.
        Classes are not available for segmentation step
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "segm",
            "classes": ["cornfields"],
        }

        # Add inputs
        user_cfg["input"] = inputs_with_classif

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_segm_on_right(self, user_cfg, inputs_with_segment):
        """
        Optimization right segmentation with validation step.
        Validation step requires both left and right segmentation.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Add inputs
        user_cfg["input"] = inputs_with_segment
        del user_cfg["input"]["left_segm"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_segm_on_left(self, user_cfg, inputs_with_segment):
        """
        Optimization left segmentation with validation step.
        Validation step requires both left and right segmentation.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Add inputs
        user_cfg["input"] = inputs_with_segment
        del user_cfg["input"]["right_segm"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_classif_on_left_with_correct_class(
        self, left_cones_classif, right_cones, user_cfg, inputs_with_classif, outputs_path
    ):
        """
        Optimization on one existing band left classification without validation step.
        """

        # Prepare the SGM configuration
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["olive tree"],
        }

        # Add inputs
        user_cfg["input"] = inputs_with_classif
        del user_cfg["input"]["right_classif"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        user_cfg = check_conf(user_cfg, pandora_machine)

        left, _ = pandora.run(pandora_machine, left_cones_classif, right_cones, user_cfg)

        # Ground truth
        gt_left = rasterio.open(outputs_path / "left_disparity_3sgm.tif").read(1)

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 2) <= 0.15

    def test_classif_on_left_and_right_with_correct_class(
        self, left_cones_classif, right_cones_classif, user_cfg, inputs_with_classif, outputs_path
    ):
        """
        Optimization on one existing band for left and right classification without validation step.
        """

        # Prepare the SGM configuration
        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["olive tree"],
        }
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_classif

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        user_cfg = check_conf(user_cfg, pandora_machine)

        left, _ = pandora.run(pandora_machine, left_cones_classif, right_cones_classif, user_cfg)

        # Ground truth
        gt_left = rasterio.open(outputs_path / "left_disparity_3sgm.tif").read(1)

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 2) <= 0.15

    def test_classif_on_right_with_correct_class(self, user_cfg, inputs_with_classif):
        """
        Optimization with right classification without validation step.
        Classification without validation step requires left classification.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["olive tree"],
        }
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Add inputs
        user_cfg["input"] = inputs_with_classif
        del user_cfg["input"]["left_classif"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_segm_on_left_without_validation(
        self, left_cones_segm, right_cones, user_cfg, inputs_with_segment, outputs_path
    ):
        """
        Optimization on left image with segmentation without validation step.
        """

        # Prepare the SGM configuration
        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_segment
        del user_cfg["input"]["right_segm"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        user_cfg = check_conf(user_cfg, pandora_machine)

        left, _ = pandora.run(pandora_machine, left_cones_segm, right_cones, user_cfg)

        # Ground truth
        gt_left = rasterio.open(outputs_path / "left_disparity_3sgm.tif").read(1)

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 2) <= 0.15

    def test_segm_on_left_and_right_without_validation(
        self, left_cones_segm, right_cones_segm, user_cfg, inputs_with_segment, outputs_path
    ):
        """
        Optimization on left and right image with segmentation without validation step.
        """

        # Prepare the SGM configuration
        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_segment

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # Check configuration
        user_cfg = check_conf(user_cfg, pandora_machine)

        left, _ = pandora.run(pandora_machine, left_cones_segm, right_cones_segm, user_cfg)

        # Ground truth
        gt_left = rasterio.open(outputs_path / "left_disparity_3sgm.tif").read(1)

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors is > 0.20, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 1) <= 0.20

        # Compares the calculated right disparity map with the ground truth
        # If the percentage of pixel errors ( error if ground truth - calculate > 2) is > 0.15, raise an error
        assert common.error(left["disparity_map"].data, gt_left, 2) <= 0.15

    def test_segm_on_right_without_validation(self, user_cfg, inputs_with_segment):
        """
        Optimization with right segmentation without validation step.
        Segmentation without validation step requires left segmentation.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]
        # Because the pipeline isn't correctly checked in this test
        user_cfg["pipeline"]["disparity"]["invalid_disparity"] = np.nan

        # Add inputs
        user_cfg["input"] = inputs_with_segment
        del user_cfg["input"]["left_segm"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_segm_on_left_and_right_with_classes(self, user_cfg, inputs_with_segment):
        """
        Optimization with right and left segmentation with classes without validation step.
        Classes are not available for segmentation step
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "segm",
            "classes": ["olive tree"],
        }
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_segment

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_segm_on_left_with_classes(self, user_cfg, inputs_with_segment):
        """
        Optimization with left segmentation with classes without validation step.
        Classes are not available for segmentation step
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a segmentation and classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "segm",
            "classes": ["olive tree"],
        }
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_segment

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_classif_on_left_with_false_classes(self, user_cfg, inputs_with_classif):
        """
        Optimization with left classification with false classes without validation step.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and false classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["pine"],
        }
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_classif
        del user_cfg["input"]["right_classif"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_classif_on_left_without_classes(self, user_cfg, inputs_with_classif):
        """
        Optimization with left classification with no classes without validation step.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and false classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "classif"}
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_classif
        del user_cfg["input"]["right_classif"]

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_classif_on_left_and_right_with_wrong_classes(self, user_cfg, inputs_with_classif):
        """
        Optimization with left and right classification with wrong classes without validation step.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and false classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {
            "source": "classif",
            "classes": ["pine"],
        }
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_classif

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_classif_on_left_and_right_without_classes(self, user_cfg, inputs_with_classif):
        """
        Optimization with left and right classification without classes without validation step.
        Check that the check_conf function raises an error.
        """

        # Prepare the SGM configuration
        # Add a classification and false classes
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "classif"}
        # Remove validation step
        del user_cfg["pipeline"]["validation"]
        del user_cfg["pipeline"]["filter.after.validation"]

        # Add inputs
        user_cfg["input"] = inputs_with_classif

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_conf(user_cfg, pandora_machine)

    def test_user_initiate_3sgm_and_validation_with_one_geomprior_segmentation(self, user_cfg, inputs_path):
        """
        Test that user can't implement 3SGM and validation if only one segmentation is given
        """

        # Prepare the SGM configuration. It contains cross_checking validation and 3SGM optimization
        # Create input cfg where only the left segmentation is present
        input_cfg = {
            "img_left": str(inputs_path / "left.png"),
            "img_right": str(inputs_path / "right.png"),
            "left_segm": str(inputs_path / "white_band_mask.png"),
            "disp_left": [-60, 0],
        }

        # Add a segmentation geometric_prior
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "segm"}

        # Create full configuration
        cfg = {"input": input_cfg, "pipeline": user_cfg["pipeline"]}

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_configuration.check_conf(cfg, pandora_machine)

    def test_user_initiate_3sgm_and_validation_with_one_geomprior_classification(self, user_cfg, inputs_path):
        """
        Test that user can't implement 3SGM and validation if only one classification is given
        """

        # Prepare the SGM configuration. It contains cross_checking validation
        # Create input cfg where only the right classification is present
        input_cfg = {
            "img_left": str(inputs_path / "left.png"),
            "img_right": str(inputs_path / "right.png"),
            "right_classif": str(inputs_path / "white_band_mask.png"),
            "disp_left": [-60, 0],
        }

        # Add a geometric_prior
        user_cfg["pipeline"]["optimization"]["geometric_prior"] = {"source": "classif"}

        # Create full configuration
        cfg = {"input": input_cfg, "pipeline": user_cfg["pipeline"]}

        # Import pandora plugins
        pandora.import_plugin()

        # Instantiate machine
        pandora_machine = PandoraMachine()

        # check the configuration
        with pytest.raises(SystemExit):
            _ = check_configuration.check_conf(cfg, pandora_machine)



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2020 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of PANDORA
#
#     https://github.com/CNES/Pandora
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module contains common functions present in Plugin_libsgm's tests.
"""

import numpy as np


def error_mask(data: np.ndarray, gt: np.ndarray) -> float:
    """
    Percentage of bad pixels ( != ground truth ) in the validity mask
    :param data: generated array in test
    :type data: np.ndarray
    :param gt: Ground truth
    :type gt: np.ndarray
    :return: Percentage
    :rtype: float
    """

    nb_rows, nb_cols = data.shape
    nb_error = 0
    for row in range(nb_rows):
        for col in range(nb_cols):
            if data[row, col] != gt[row, col]:
                nb_error += 1

    return nb_error / float(nb_rows * nb_cols)


def strict_error(data: np.ndarray, gt: np.ndarray) -> float:
    """
    Average of bad pixels  ( != ground truth )
    :param data: generated array in test
    :type data: np.ndarray
    :param gt: Ground truth
    :type gt: np.ndarray
    :return: Percentage
    :rtype: float
    """

    nb_rows, nb_cols = data.shape
    nb_error = 0
    for row in range(nb_rows):
        for col in range(nb_cols):
            if data[row, col] != gt[row, col]:
                nb_error += 1

    return nb_error / float(nb_rows * nb_cols)


def error(
    data: np.ndarray, gt: np.ndarray, threshold: int, unknown_disparity: int = 0, flag_inverse_value: bool = True
) -> float:
    """
    Percentage of bad pixels whose error is > threshold
    If the GT is tested : the values for the left image might be inverse

    :param data: generated array in test
    :type data: np.ndarray
    :param gt: Ground truth
    :type gt: np.ndarray
    :param threshold: accepted threshold in test
    :type threshold: int
    :param unknown_disparity: value for invalid disp in GT
    :type unknown_disparity: int
    :param flag_inverse_value: Whether the GT and disparity map values are reversed
    :type flag_inverse_value: bool
    :return: Percentage
    :rtype: float
    """

    if flag_inverse_value:
        data = -1 * data
    nb_row, nb_col = data.shape
    nb_error = 0
    for row in range(nb_row):
        for col in range(nb_col):
            if gt[row, col] != unknown_disparity:
                if abs((data[row, col] + gt[row, col])) > threshold:
                    nb_error += 1

    return nb_error / float(nb_row * nb_col)



#!/usr/bin/env python
# coding: utf8
#
# Copyright (c) 2020 Centre National d'Etudes Spatiales (CNES).
#
# This file is part of Pandora plugin LibSGM
#
#     https://github.com/CNES/Pandora_plugin_libsgm
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
This module provides class and functions to compute penalties used to optimize the cost volume using the LibSGM library
"""

import logging
import sys
from abc import ABCMeta, abstractmethod
from typing import Tuple, List, Union

import numpy as np
import xarray as xr


class AbstractPenalty:
    """
    Penalty abstract class
    """

    __metaclass__ = ABCMeta

    penalty_methods_avail = {}

    def __new__(
        cls, directions: List[List[int]], **cfg: Union[str, int, float, bool]
    ):  # pylint: disable=unused-argument
        """
        Return the plugin associated with the penalty_method given in the configuration

        :param directions: directions to
        :type directions: list of [x offset, y offset]
        :param cfg: configuration {'penalty_method': value}
        :type cfg: dictionary
        """
        if cls is AbstractPenalty:
            if isinstance(cfg["penalty_method"], str):
                try:
                    return super(AbstractPenalty, cls).__new__(cls.penalty_methods_avail[cfg["penalty_method"]])
                except KeyError:
                    logging.error("No penalty method named % supported", cfg["penalty_method"])
                    sys.exit(1)
            else:
                if isinstance(cfg["penalty_method"], unicode):  # type: ignore # pylint: disable=undefined-variable
                    # creating a plugin from registered short name given as unicode (py2 & 3 compatibility)
                    try:
                        return super(AbstractPenalty, cls).__new__(
                            cls.penalty_methods_avail[cfg["penalty_method"].encode("utf-8")]
                        )
                    except KeyError:
                        logging.error("No penalty method named % supported", cfg["penalty_method"])
                        sys.exit(1)
        else:
            return super(AbstractPenalty, cls).__new__(cls)

    @classmethod
    def register_subclass(cls, short_name: str, *args: str):
        """
        Allows to register the subclass with its short name

        :param short_name: the subclass to be registered
        :type short_name: string
        :param args: allows to register one plugin that contains different methods
        :param args: string
        """

        def decorator(subclass):
            """
            Registers the subclass in the available methods

            :param subclass: the subclass to be registered
            :type subclass: object
            """
            cls.penalty_methods_avail[short_name] = subclass
            for arg in args:
                cls.penalty_methods_avail[arg] = subclass
            return subclass

        return decorator

    @abstractmethod
    def desc(self):
        """
        Describes the penalty method

        """
        print("Penalty method description")

    @abstractmethod
    def compute_penalty(
        self, cv: xr.Dataset, img_left_array: np.ndarray, img_right_array: np.ndarray
    ) -> Tuple[float, np.ndarray, np.ndarray]:
        """
        Compute penalty

        :param cv: the cost volume, with the data variables:

            - cost_volume 3D xarray.DataArray (row, col, disp)
            - confidence_measure 3D xarray.DataArray (row, col, indicator)
        :type cv: xarray.Dataset
        :param img_left_array: left  image
        :type img_left_array: np.ndarray
        :param img_right_array: right  image
        :type img_right_array: np.ndarray
        :return: invalid values, P1 and P2 penalties
        :rtype: tuple(float, numpy array, numpy array)
        """

    @staticmethod
    def compute_gradient(img: np.ndarray, direction: List[int]) -> np.ndarray:
        """
        Compute image gradient

        :param img: image
        :type img: numpy array of shape(n,m)
        :param direction: directions to
        :type direction: list of [x offset, y offset]
        :return: Gradient
        :rtype: numpy array of shape(n-dir[0], m-dir[1])
        """
        mat1 = img[
            max(direction[0], 0) : min(img.shape[0] + direction[0], img.shape[0]),
            max(direction[1], 0) : min(img.shape[1] + direction[1], img.shape[1]),
        ]
        mat2 = img[
            max(-direction[0], 0) : min(img.shape[0] - direction[0], img.shape[0]),
            max(-direction[1], 0) : min(img.shape[1] - direction[1], img.shape[1]),
        ]

        return np.abs(mat1 - mat2)
